==> blobAndConquer-0.1-1.tar.gz: BlobAndConquer/src/triggers.cpp <==
/*
Copyright (C) 2005 Parallel Realities

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include "triggers.h"

void addTrigger(Properties *props)
{
	Trigger *trigger = mission->spawnTrigger();

	trigger->type = engine->getValueOfDefine(props->getString("type", "type"));
	trigger->action = engine->getValueOfDefine(props->getString("action", "action"));
	
	trigger->load(props);
}

void performMainTriggerAction(Trigger *trigger)
{
	debug(("performMainTriggerAction() - ['%s', '%s']\n", trigger->name.getText(), trigger->targetName.getText()));
	
	switch (trigger->action)
	{
		case TRIGGER_ACTION_COMPLETE_OBJECTIVE:
			updateObjectiveStatus(trigger->objectiveId, OBJ_COMPLETE);
			break;
			
		case TRIGGER_ACTION_FAIL_OBJECTIVE:
			updateObjectiveStatus(trigger->objectiveId, OBJ_FAILED);
			break;
			
		case TRIGGER_ACTION_ENABLE_OBJECTIVE:
			enableObjective(trigger->objectiveId);
			break;
			
		case TRIGGER_ACTION_UPDATE_OBJECTIVE:
			if (updateObjectiveCurrentValue(trigger->objectiveId))
			{
				trigger->active = false;
			}
			break;
			
		case TRIGGER_ACTION_ENABLE_FEATURE:
			enableFeature(trigger->targetName.getText(), 1);
			break;
			
		case TRIGGER_ACTION_DISABLE_FEATURE:
			enableFeature(trigger->targetName.getText(), 0);
			break;
			
		case TRIGGER_ACTION_TOGGLE_STRUCTURE:
			printf("%s\n", trigger->targetName.getText());
			handleStructureStateChange(trigger->targetName.getText(), true);
			break;
	}
}

void triggerCheckLocation(Trigger *trigger)
{
	/*
	if (Math::getDistance(trigger->position.x, trigger->position.y, player->position.x, player->position.y) < trigger->radius)
	{
		triggerActivateAllInGroup(trigger->name.getText());
	}
	
	trigger->thinkTime = 0;
	*/
}

void fireTriggers(char *name, int type)
{
	debug(("fireTriggers() - '%s' (%d)\n", name, type));
	
	if (strcmp(name, "") == 0)
	{
		return;
	}
	
	for (Trigger *trigger = (Trigger*)mission->triggerList.getFirstElement() ; trigger != NULL ; trigger = (Trigger*)trigger->next)
	{
		if (!trigger->active)
		{
			continue;
		}
		
		if ((trigger->type != type) && (type != TRIGGER_TYPE_ANY))
		{
			continue;
		}
		
		if (trigger->name == name)
		{
			trigger->deactivate();
			performMainTriggerAction(trigger);
		}
	}
}

void doTriggers()
{
	glEnable(GL_BLEND);
	glDisable(GL_TEXTURE_2D);
	
	GLColor c;
	
	for (Trigger *trigger = (Trigger*)mission->triggerList.getFirstElement() ; trigger != NULL ; trigger = (Trigger*)trigger->next)
	{
		if (trigger->type != TRIGGER_TYPE_LOCATION)
		{
			continue;
		}
		
		if (trigger->active)
		{
			triggerCheckLocation(trigger);
		}
		
		if (!trigger->invisible)
		{
			c = GLColor::green.lighter();
			c.color[3] = 0.8;
			
			if (trigger->radius > 0)
			{
				drawCyclinder(trigger->position, 8, trigger->radius, 25, c);
			}
			
			if (!trigger->active)
			{
				trigger->radius -= (engine->getTimeDifference(TD_LOGIC) * 0.1);
			
				if (trigger->radius == 0)
				{
					trigger = (Trigger*)trigger->previous;
					mission->triggerList.remove(trigger->next);
				}
			}
		}
	}
}

==> blobAndConquer-0.2-1.tar.gz: BlobAndConquer/src/triggers.cpp <==
/*
Copyright (C) 2005 Parallel Realities

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include "triggers.h"

void addTrigger(Properties *props)
{
	Trigger *trigger = mission->spawnTrigger();

	trigger->type = engine->getValueOfDefine(props->getString("type", "type"));
	trigger->action = engine->getValueOfDefine(props->getString("action", "action"));
	
	trigger->load(props);
}

void performMainTriggerAction(Trigger *trigger)
{
	debug(("performMainTriggerAction() - ['%s', '%s']\n", trigger->name.getText(), trigger->targetName.getText()));
	
	switch (trigger->action)
	{
		case TRIGGER_ACTION_COMPLETE_OBJECTIVE:
			updateObjectiveStatus(trigger->objectiveId, OBJ_COMPLETE);
			break;
			
		case TRIGGER_ACTION_FAIL_OBJECTIVE:
			updateObjectiveStatus(trigger->objectiveId, OBJ_FAILED);
			break;
			
		case TRIGGER_ACTION_ENABLE_OBJECTIVE:
			enableObjective(trigger->objectiveId);
			break;
			
		case TRIGGER_ACTION_UPDATE_OBJECTIVE:
			if (updateObjectiveCurrentValue(trigger->objectiveId))
			{
				trigger->active = false;
			}
			break;
			
		case TRIGGER_ACTION_ENABLE_FEATURE:
			enableFeature(trigger->targetName.getText(), 1);
			break;
			
		case TRIGGER_ACTION_DISABLE_FEATURE:
			enableFeature(trigger->targetName.getText(), 0);
			break;
			
		case TRIGGER_ACTION_TOGGLE_STRUCTURE:
			handleStructureStateChange(trigger->targetName.getText(), true);
			break;
			
		case TRIGGER_ACTION_TELEPORT_BLOB:
			teleportOutBlob(trigger->targetName.getText());
			break;
			
		case TRIGGER_ACTION_USE_CAMERA:
			activateInGameCutscene(trigger->targetName.getText(), 100);
			break;
	}
}

void triggerCheckLocation(Trigger *trigger)
{
	/*
	if (Math::getDistance(trigger->position.x, trigger->position.y, player->position.x, player->position.y) < trigger->radius)
	{
		triggerActivateAllInGroup(trigger->name.getText());
	}
	
	trigger->thinkTime = 0;
	*/
}

void fireTriggers(char *name, int type)
{
	debug(("fireTriggers() - '%s' (%d)\n", name, type));
	
	if (strcmp(name, "") == 0)
	{
		return;
	}
	
	// first check to see if a camera is part of the group... If it is then activate it first
	
	for (Trigger *trigger = (Trigger*)mission->triggerList.getFirstElement() ; trigger != NULL ; trigger = (Trigger*)trigger->next)
	{
		if (!trigger->active)
		{
			continue;
		}
		
		if ((trigger->type != type) && (type != TRIGGER_TYPE_ANY))
		{
			continue;
		}
		
		if (trigger->name == name)
		{
			if (trigger->action == TRIGGER_ACTION_USE_CAMERA)
			{
				debug(("fireTriggers() - Doing camera first...\n"));
				trigger->active = false;
				performMainTriggerAction(trigger);
				return;
			}
		}
	}
	
	debug(("fireTriggers() - No camera in group...\n"));
	
	for (Trigger *trigger = (Trigger*)mission->triggerList.getFirstElement() ; trigger != NULL ; trigger = (Trigger*)trigger->next)
	{
		if (!trigger->active)
		{
			continue;
		}
		
		if ((trigger->type != type) && (type != TRIGGER_TYPE_ANY))
		{
			continue;
		}
		
		if (trigger->name == name)
		{
			if ((trigger->action != TRIGGER_ACTION_UPDATE_OBJECTIVE) && (trigger->action != TRIGGER_ACTION_FAIL_OBJECTIVE))
			{
				trigger->active = false;
			}
			
			performMainTriggerAction(trigger);
		}
	}
}

void doTriggers()
{
	glEnable(GL_BLEND);
	glDisable(GL_TEXTURE_2D);
	
	GLColor c;
	
	for (Trigger *trigger = (Trigger*)mission->triggerList.getFirstElement() ; trigger != NULL ; trigger = (Trigger*)trigger->next)
	{
		if (trigger->type != TRIGGER_TYPE_LOCATION)
		{
			continue;
		}
		
		if (trigger->active)
		{
			triggerCheckLocation(trigger);
		}
		
		if (!trigger->invisible)
		{
			c = GLColor::green.lighter();
			c.color[3] = 0.8;
			
			if (trigger->radius > 0)
			{
				drawCyclinder(trigger->position, 8, trigger->radius, 25, c);
			}
			
			if (!trigger->active)
			{
				trigger->radius -= (engine->getTimeDifference(TD_LOGIC) * 0.1);
			
				if (trigger->radius == 0)
				{
					trigger = (Trigger*)trigger->previous;
					mission->triggerList.remove(trigger->next);
				}
			}
		}
	}
}
